"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const domtokenlist_1 = require("./domtokenlist");
class DOMNode {
    constructor(tagName, parent, metaTable, location) {
        this.children = [];
        this.tagName = tagName;
        this.parent = parent;
        this.attr = {};
        this.meta = metaTable ? metaTable.getMetaFor(tagName) : null;
        this.open = true;
        this.closed = false;
        this.selfClosed = false;
        this.voidElement = this.meta ? this.meta.void : false;
        this.location = location;
        if (parent) {
            parent.children.push(this);
        }
    }
    static rootNode() {
        return new DOMNode(undefined, undefined);
    }
    static fromTokens(startToken, endToken, parent, metaTable) {
        const tagName = startToken.data[2];
        if (!tagName) {
            throw new Error("tagName cannot be empty");
        }
        const open = startToken.data[1] !== '/';
        const node = new DOMNode(tagName, open ? parent : undefined, metaTable, startToken.location);
        node.selfClosed = endToken.data[0] === '/>';
        node.open = open;
        node.closed = node.selfClosed || node.voidElement;
        return node;
    }
    is(tagName) {
        return (this.tagName && tagName === '*') || this.tagName === tagName;
    }
    isRootElement() {
        return typeof this.tagName === 'undefined';
    }
    setAttribute(key, value) {
        key = key.toLowerCase();
        this.attr[key] = value;
    }
    hasAttribute(key) {
        key = key.toLowerCase();
        return key in this.attr;
    }
    getAttribute(key) {
        key = key.toLowerCase();
        if (key in this.attr) {
            return this.attr[key];
        }
        else {
            return null;
        }
    }
    append(node) {
        this.children.push(node);
    }
    get classList() {
        return new domtokenlist_1.DOMTokenList(this.getAttribute('class'));
    }
    get siblings() {
        return this.parent.children;
    }
    getElementsByTagName(tagName) {
        return this.children.reduce(function (matches, node) {
            return matches.concat(node.is(tagName) ? [node] : [], node.getElementsByTagName(tagName));
        }, []);
    }
    visitDepthFirst(callback) {
        function visit(node) {
            node.children.forEach(visit);
            if (!node.isRootElement()) {
                callback(node);
            }
        }
        visit(this);
    }
    someChildren(callback) {
        return this.children.some(visit);
        function visit(node) {
            if (callback(node)) {
                return true;
            }
            else {
                return node.children.some(visit);
            }
        }
    }
    everyChildren(callback) {
        return this.children.every(visit);
        function visit(node) {
            if (!callback(node)) {
                return false;
            }
            return node.children.every(visit);
        }
    }
    find(callback) {
        function visit(node) {
            if (callback(node)) {
                return node;
            }
            for (const child of node.children) {
                const match = child.find(callback);
                if (match) {
                    return match;
                }
            }
            return null;
        }
        return visit(this);
    }
}
exports.DOMNode = DOMNode;
