"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const parser_1 = require("./parser");
const reporter_1 = require("./reporter");
const lexer_1 = require("./lexer");
const token_1 = require("./token");
const fs = require('fs');
class HtmlValidate {
    constructor(options) {
        this.config = config_1.default.fromObject(options || {});
    }
    string(str) {
        return this.parse({ data: str, filename: 'inline' });
    }
    file(filename, mode) {
        const text = fs.readFileSync(filename, { encoding: 'utf8' });
        const source = { data: text, filename };
        switch (mode) {
            case 'lint':
            case undefined:
                return this.parse(source);
            case 'dump-tokens':
                return this.dumpTokens(source);
            default:
                throw new Error(`Unknown mode "${mode}"`);
        }
    }
    parse(src) {
        const report = new reporter_1.Reporter();
        const rules = this.config.getRules();
        const parser = this.getParser();
        for (const name in rules) {
            const data = rules[name];
            this.loadRule(name, data, parser, report);
        }
        try {
            parser.parseHtml(src);
        }
        catch (e) {
            if (e instanceof lexer_1.InvalidTokenError) {
                report.addManual(e.location.filename, {
                    ruleId: undefined,
                    severity: config_1.default.SEVERITY_ERROR,
                    message: e.message,
                    line: e.location.line,
                    column: e.location.column,
                });
            }
            else {
                throw e;
            }
        }
        return report.save();
    }
    getParser() {
        return new parser_1.default(this.config);
    }
    dumpTokens(source) {
        const lexer = new lexer_1.Lexer();
        for (const token of lexer.tokenize(source)) {
            const data = token.data ? token.data[0] : null;
            process.stdout.write(`TOKEN: ${token_1.TokenType[token.type]}
  Data: ${JSON.stringify(data)}
  Location: ${token.location.filename}:${token.location.line}:${token.location.column}
`);
        }
        return {
            valid: true,
            results: [],
        };
    }
    loadRule(name, data, parser, report) {
        const [severity, options] = data;
        if (severity >= config_1.default.SEVERITY_WARN) {
            let rule;
            try {
                rule = require(`./rules/${name}`);
            }
            catch (e) {
                rule = {
                    name: name,
                    init: (parser) => {
                        parser.on('dom:load', (event, report) => {
                            report(null, `Definition for rule '${name}' was not found`);
                        });
                    },
                };
            }
            rule.init(this.createProxy(parser, rule, report), options);
        }
    }
    createProxy(parser, rule, report) {
        return {
            on: function (event, callback) {
                parser.on(event, function (event, data) {
                    callback.call(rule, data, reportFunc);
                    function reportFunc(node, message, location) {
                        report.add(node, rule, message, location || data.location || node.location);
                    }
                });
            },
        };
    }
}
exports.default = HtmlValidate;
